Software testing in tpcclib project
===================================

This page describes the process of software testing

CTest, included in [CMake](http://www.cmake.org), is used in automatic testing, 
and all tests are [executed](#run) with one command after the library is built.

## Tests for the library functions ##

C source codes for testing are included in the source codes of the library,
and compiled as the project executables are compiled, and executed as explained in `README.md`.

Running these tests do not require any special settings on the platform, but the directory of 
the compiled executables should be in the search path. 

\htmlonly
<img src="http://www.turkupetcentre.net/pic/iwillalwaystestlibraryfunctions.gif"/>
\endhtmlonly 

## Tests for executables ##

Since executables (binaries) are based on library functions, which have already been tested (in v2), 
the tests for executables are not very detailed.

Very basic testing is first performed using CMake commands, verifying that executables can be found 
and run, and that they provide usage information with appropriate command-line options.

Running these tests do not require any special settings on the platform. However, library 
applications must reside in a folder included in the systems search path for the application test 
scripts to work. 

In the crucial tests the executables are tested to perform as expected in normal use, and therefore 
tests are run using multi-platform scripts that mimic the real use cases. Note that these tests are 
not intended to test whether the applied method is suitable to retrieve meaningful outcome from
any specific data set; that is considered as science, and out of the scope of software development. 

These tests are based on `bash` scripts, located in `test` folder under the source directories. 
CMake is used to execute these test scripts automatically. In Windows platform this requires that 
the MinGW/MSYS bin folders (containing for example bash.exe, and other bash command executables) 
are included in Windows path. When correctly installed, you should be able to open Windows
command prompt window (`cmd.exe`), and run there for example the commands

    gcc --version
    pwd
    ls -al

If test script is not ready at the time when application itself is added, then create at least 
a dummy script which returns a non-zero value and prints to stdout 'test not yet made' or something 
like that.

## <a name="run"></a>Running tests ##

After successful `make install`, you can run the tests. However, application tests are dependent on 
the applications of the tpcclib. Therefore, before running the tests, you should have previous 
versions of tpcclib applications in your computers search path, or you should copy the `./bin/` 
contents to a folder that is in the path.

Running tests may take a long time. Recommended command to run the tests in quad-core computer is

    ctest --output-on-failure --parallel 4

because then CTest uses 4 threads for testing (you can change the number according to your 
computer), and prints to screen all output from those tests only that failed.

Alternatively, you can run

    ctest -V

to get more verbose output from (the same) tests; `ctest` is included in CMake installation. 
Do not use `--parallel` option here, because it would mess up the verbose output.
