Things to do
============

## Next ##

- Update compile instructions to include MSYS2 and Gitlab.
- Update readme and other instructions to include links to binary releases.
- Add __attribute__((fallthrough)); to all switch fallthrough cases, after
  all compilers support that. 
- Check that the new fit_sinf can be used in TAF rat studies, and in 
  O2-simulation.
- Test interpolation functions using realistic PET TTAC functions, including
  gaps and overlaps.
- Make interpolation and integration functions that consider frames as step
  functions; it will keep frames independent on each other.
- Test the quantitativity of BF-programs with very wide range of frames.
- Add support for NIfTI-2, see <http://nifti.nimh.nih.gov/nifti-2/>
- Add rFRTM by Mandeville et al (2016).

## Bugs to fix

- tacstudy was able to mess up result file with option --force.
- inpstart did not feel necessary to fix input tac which was not accepted by 
  patlak.
- PMOD BLD TAC names contain extra '_' in the end; cannot replicate!

## Tests

- Use CMakeLists.txt in v2/libtpctac and v1/ecat as templates for others;
  then updated test data and scripts will be automatically updated during build. 

## Applications ##

- Move necessary applications from the old TPC C libraries to v1 or v2.
- rescoll should have option to save data as CSV table(s).

## General library functions ##

- Edit statusSet() in v2 so that it updates previous line number and error code 
  only if function changes.

## General ##

- Required: C99 and basic POSIX compatibility. That means also that
  Visual Studio is not supported (I guess).
- Make sure everything is thread safe in v2.
- User info must be compatible with Harri's script interface.
- Errors etc into common struct from all functions?
- Rename all functions, globals, and most structs with tpc*.

## C99 and POSIX ##

- Structs can be copied directly with `=` but pointer addresses are copied too
  so pointer contents must be copied with for().
- Struct contents are automatically initiated to 0 and NULL; make sure that it
  works in Windows, Linux, and OS X.
- Forget about own `strncasecmp()`, `strdup()`, etc.
- Struct tag and typedef can (and should) be the same:
  `typedef struct TAC {int a; double b;} TAC;`

## Security ##

- Replace all `sprintf()`'s with `snprintf()`, or `(v)asprintf()`
  (from _GNU_SOURCE).
- Replace strncpy() with strlcpy(); automatically NULL terminated.
- Replace strncat() with strlcat(); automatically NULL terminated.
- Replace all `strtok()` calls with something else because `strtok()` is not
  thread-safe (unless it becomes thread-safe with new compiler versions).

## Packages ##

- 32 vs 64 bit works now at least in Windows/MSYS/MinGW-w64 with command-line
  options (see Doxygen documentation); check later with future versions of CMake
  and NSIS if these start to work with less command-line options.
- CMake should now support official package builder of OS X
  (productbuild); try that to build packages for OS X.

## Doxygen ##

- Follow the instructions in [codingpractice.md](codingpractice.md).
