#!/bin/bash
#: Title      : test_imgfiltg
#: Date       : 2018-06-04
#: Author     : "Vesa Oikonen" <vesa.oikonen@utu.fi>
#: Options    : None

# Set the name of executable to test
if [ -n "${OS+1}" ] && [ "$OS" = "Windows_NT" ]; then
EXT=.exe;
else
EXT=;
fi
PROGRAM=../../imgfiltg$EXT;
if [ ! -f $PROGRAM ]; then
  printf "Failed: executable does not exist.\n"
  exit 1
fi


printf "=====================================================================\n"
printf "creating test data for %s\n" $PROGRAM
printf "=====================================================================\n"


printf "\n verifying that required data exists \n"

if [ ! -f cross.header ] || [ ! -f cross.dat ] || [ ! -f cross_tiny.dat ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi
if [ ! -f square.header ] || [ ! -f square.dat ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi
if [ ! -f frame.dat ] || [ ! -f imgfiltg.dft ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi
if [ ! -f cross_octave_smoothed_4.dat ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi
if [ ! -f cross_octave_smoothed_8.dat ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi
if [ ! -f cross_tiny_octave_smoothed_2.dat ]; then
  printf "Failed: required data does not exist.\n"
  exit 1
fi


printf "\n making test data \n"

if [ ! -f cross.v ]; then
  asc2flat cross.dat cross.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -zoom=2.5 -scanner=HR+ cross.bin cross.v 1 1 100 100 
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr cross.v cross.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml cross.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt cross.v 2009-12-18 14:17:04
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cross.v cross.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f cross_tiny.v ]; then
  asc2flat cross_tiny.dat cross_tiny.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -zoom=2.5 -scanner=HR+ cross_tiny.bin cross_tiny.v 1 1 12 12 
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr cross_tiny.v cross.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml cross_tiny.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt cross_tiny.v 2009-12-21 11:58:32
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cross_tiny.v cross_tiny.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi


if [ ! -f dynamic_tiny.v ] || [ ! -f dynamic.v ] || [ ! -f dynamic.img ] || [ ! -f static.v ]; then
  rm -f ana/dynamic* nii/dynamic* static.v static_tiny.v dynamic3d_tiny.v
  dft2img -scanner=HR+ -zoom=2 -dim=12 imgfiltg.dft dynamic_tiny.v 
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  dft2img -scanner=HR+ -zoom=2 -dim=128 imgfiltg.dft dynamic.v 
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml dynamic.v dynamic_tiny.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt dynamic.v 2005-11-20 18:50:00
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt dynamic_tiny.v 2009-12-16 18:03:00
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7to63 dynamic.v dynamic.img
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f ana/dynamic.img ] || [ ! -f ana/dynamic.hdr ] || [ ! -f ana/dynamic.sif ]; then
  rm -f ana/*
  ecat2ana -sif -o=ana dynamic.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f nii/dynamic.nii ] || [ ! -f nii/dynamic.sif ]; then
  rm -f nii/*
  ecat2nii -sif -o=nii dynamic.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f static.v ] || [ ! -f static_tiny.v ]; then
  imginteg -avg dynamic.v 0 0 static.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imginteg -avg dynamic_tiny.v 0 0 static_tiny.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f dynamic3d_tiny.v ]; then
  imgadd -x=15 dynamic3d_tiny.v dynamic_tiny.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi


if [ ! -f cube.v ] || [ ! -f square.v ]; then
  asc2flat square.dat square.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -scanner=hrrt -zoom=2.5 square.bin square.v 1 1 100 100
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr square.v square.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml square.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt square.v 2013-03-07 10:37:34
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgcalc square.v x 0 empty.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgadd -x=5 temp1.v empty.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgadd -x=7 temp2.v temp1.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgadd -x=5 temp1.v square.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgadd -x=6 temp3.v temp1.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgadd cube.v temp2.v temp3.v temp2.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cube.v cube.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi



printf "\n\n Making comparison data based on smoothing in Octave \n\n"

if [ ! -f cross_octave_smoothed_4.v ]; then
  asc2flat cross_octave_smoothed_4.dat temp.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -scanner=hr+ -zoom=2.5 temp.bin temp.v 1 1 100 100
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr temp.v cross_octave.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgcalc temp.v x 100 cross_octave_smoothed_4.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml cross_octave_smoothed_4.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt cross_octave_smoothed_4.v 2009-12-18 14:17:04
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cross_octave_smoothed_4.v cross_octave_smoothed_4.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f cross_octave_smoothed_8.v ]; then
  asc2flat cross_octave_smoothed_8.dat temp.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -scanner=hr+ -zoom=2.5 temp.bin temp.v 1 1 100 100
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr temp.v cross_octave.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgcalc temp.v x 100 cross_octave_smoothed_8.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml cross_octave_smoothed_8.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt cross_octave_smoothed_8.v 2009-12-28 13:31:00
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cross_octave_smoothed_8.v cross_octave_smoothed_8.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi

if [ ! -f cross_tiny_octave_smoothed_2.v ]; then
  asc2flat cross_tiny_octave_smoothed_2.dat temp.bin
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  flat2img -scanner=hr+ -zoom=2.5 temp.bin temp.v 1 1 12 12
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  e7evhdr temp.v cross_octave.header
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgcalc temp.v x 100 cross_tiny_octave_smoothed_2.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  imgunit -us=kBq/ml cross_tiny_octave_smoothed_2.v
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  esetstrt cross_tiny_octave_smoothed_2.v 2009-12-21 11:58:32
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
  img2tif -rb -s cross_tiny_octave_smoothed_2.v cross_tiny_octave_smoothed_2.tif
  if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
fi




printf "\n\n"
printf "=====================================================================\n"
printf "testing %s\n" $PROGRAM
printf "=====================================================================\n"

printf "\n 1.0.0 \n"
printf " Test case: Set SD to zero. \n"
printf " Expected result 1: Warning. \n"
printf " Expected result 2: Output image is the same as input image. \n\n"

rm -f filtered.v stdout.txt
imgprofi -pxl=1,1,1 cross_tiny.v xo.pro yo.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

$PROGRAM cross_tiny.v filtered.v 0 &> stdout.txt
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
if [ ! -f filtered.v ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Warning" "with given SD no filtering is applied."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 filtered.v x.pro y.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg -x1=-8 -x2=8 f0crossx.svg xo.pro x.pro
tac2svg -x1=-8 -x2=8 f0crossy.svg yo.pro y.pro

imgmatch cross_tiny.v filtered.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 1.1.0 \n"
printf " Test case: Set SD to 8 pixels with default settings. \n"
printf " Expected result 1: Profile integrals are the same after smoothing. \n"
printf " Expected result 2: Correct output image as compared to Octave. \n\n"

rm -f cross_imgfiltg_8.v cross_imgfiltg_8.tif
imgprofi -pxl=1,1,1 cross.v xo.pro yo.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgprofi -pxl=1,1,1 cross_octave_smoothed_8.v xc.pro yc.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

$PROGRAM cross.v cross_imgfiltg_8.v 8pxl
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s cross_imgfiltg_8.v cross_imgfiltg_8.tif
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 cross_imgfiltg_8.v x.pro y.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossx.svg xo.pro x.pro -l xc.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossy.svg yo.pro y.pro -l yc.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

interpol -i -x=100 xo.pro test1.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
interpol -i -x=100 x.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n x passed. \n\n"

interpol -i -x=100 yo.pro test1.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
interpol -i -x=100 y.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n y passed. \n\n"

imgmatch -abs=0.1 -rel=0.1 cross_imgfiltg_8.v cross_octave_smoothed_8.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"



printf "\n 1.1.1 \n"
printf " Test case 1: Set SD to 8 mm. \n"
printf " Test case 2: options to set pixel and filter sizes. \n"
printf " Expected result 1: Profile integrals are the same after smoothing. \n"
printf " Expected result 2: Correct output image as compared to Octave. \n\n"

rm -f cross_imgfiltg_8mm.v cross_imgfiltg_8mm.tif
rm -f xmm.pro ymm.pro

$PROGRAM -pixelsize=1 -fsize=91mm cross.v cross_imgfiltg_8mm.v 8mm
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s cross_imgfiltg_8mm.v cross_imgfiltg_8mm.tif
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 cross_imgfiltg_8mm.v xmm.pro ymm.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossxmm.svg xo.pro x.pro -l xmm.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossymm.svg yo.pro y.pro -l ymm.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

interpol -i -x=100 xmm.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n x passed. \n\n"

interpol -i -x=100 ymm.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n y passed. \n\n"

imgmatch -abs=0.5 -rel=0.2 cross_imgfiltg_8mm.v cross_octave_smoothed_8.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"



printf "\n 2.0.0 \n"
printf " Test case 1: Optional fast2D algorithm. \n"
printf " Test case 2: Set SD to zero. \n"
printf " Expected result 1: Warning. \n"
printf " Expected result 2: Output image is the same as input image. \n\n"

rm -f filtered.v stdout.txt
imgprofi -pxl=1,1,1 cross_tiny.v xo.pro yo.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

$PROGRAM -method=fast2d cross_tiny.v filtered.v 0 &> stdout.txt
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
if [ ! -f filtered.v ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Warning" "with given SD no filtering is applied."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 filtered.v x.pro y.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg -x1=-8 -x2=8 f0crossx.svg xo.pro x.pro
tac2svg -x1=-8 -x2=8 f0crossy.svg yo.pro y.pro

imgmatch cross_tiny.v filtered.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.1.0 \n"
printf " Test case 1: Optional fast2D algorithm. \n"
printf " Test case 2: Set SD to 8 pixels with step setting. \n"
printf " Note: This is approximate method therefore testing is not strict either. \n"
printf " Expected result 1: Profile integrals are about the same after smoothing. \n"
printf " Expected result 2: Close to correct output image as compared to Octave. \n\n"

rm -f cross_imgfiltg_8.v cross_imgfiltg_8.tif
imgprofi -pxl=1,1,1 cross.v xo.pro yo.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgprofi -pxl=1,1,1 cross_octave_smoothed_8.v xc.pro yc.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

$PROGRAM -method=fast2d -steps=40 cross.v cross_imgfiltg_8.v 8pxl
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s cross_imgfiltg_8.v cross_imgfiltg_8.tif
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 cross_imgfiltg_8.v x.pro y.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossx.svg xo.pro x.pro -l xc.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossy.svg yo.pro y.pro -l yc.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

interpol -i -x=100 xo.pro test1.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
interpol -i -x=100 x.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 -abs=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n x passed. \n\n"

interpol -i -x=100 yo.pro test1.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
interpol -i -x=100 y.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 -abs=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n y passed. \n\n"

imgmatch -abs=1.0 -rel=1.0 cross_imgfiltg_8.v cross_octave_smoothed_8.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 2.1.1 \n"
printf " Test case 1: Optional fast2D algorithm. \n"
printf " Test case 2: Set SD to 8 mm. \n"
printf " Test case 2: options to set pixel and filter sizes and step number. \n"
printf " Note: This is approximate method therefore testing is not strict either. \n"
printf " Expected result 1: Profile integrals are about the same after smoothing. \n"
printf " Expected result 2: Close to correct output image as compared to Octave. \n\n"

rm -f cross_imgfiltg_8mm.v cross_imgfiltg_8mm.tif
rm -f xmm.pro ymm.pro

$PROGRAM -method=fast2d -steps=40 -pixelsize=1 -fsize=91mm cross.v cross_imgfiltg_8mm.v 8mm
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=1,1,1 cross_imgfiltg_8mm.v xmm.pro ymm.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossxmm.svg xo.pro x.pro -l xmm.pro 
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tac2svg crossymm.svg yo.pro y.pro -l ymm.pro
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

interpol -i -x=100 xmm.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n x passed. \n\n"

interpol -i -x=100 ymm.pro test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -rel=0.1 test1.dat test2.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n y passed. \n\n"

imgmatch -abs=1.0 -rel=1.0 cross_imgfiltg_8mm.v cross_octave_smoothed_8.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"




printf "\n 3.0.0 \n"
printf " Test case 1: Optional fast3D algorithm. \n"
printf " Test case 2: Set SD to zero. \n"
printf " Expected result 1: Warning. \n"
printf " Expected result 2: Output image is the same as input image. \n\n"

rm -f cube_filtered.v stdout.txt

$PROGRAM -method=fast3d cube.v cube_filtered.v 0 &> stdout.txt
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
if [ ! -f cube_filtered.v ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Warning" "with given SD no filtering is applied."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgmatch cube.v cube_filtered.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 3.1.0 \n"
printf " Test case 1: Optional fast3D algorithm. \n"
printf " Test case 2: Reasonable SD and steps. \n"
printf " Expected result: Profiles are the same from all directions. \n\n"

rm -f cube_filtered.v

$PROGRAM -method=fast3d -steps=20 cube.v cube_filtered.v 5
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s cube_filtered.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgmatch -abs=0.1 -rel=0.1 cube.v cube_filtered.v
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgflip -right cube_filtered.v cube_filtered_right.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgprofi -pxl=50,50,50 cube_filtered.v profilex.dat profiley.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
imgprofi -pxl=50,50,50 cube_filtered_right.v profilez.dat profiley.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
tacren profilex.dat 1 X__
tacren profiley.dat 1 Y__
tacren profilez.dat 1 Z__
tac2svg -legend=y -mt="Profiles of filtered cube" cube_filtered_profiles.svg -l profilex.dat profiley.dat profilez.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -x=y -y=y -tacnames=n -xunit=y -yunit=y -abs=0.1 -rel=0.1 profilex.dat profilez.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -x=y -y=y -tacnames=n -xunit=y -yunit=y -abs=0.1 -rel=0.1 profiley.dat profilez.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 3.1.1 \n"
printf " Test case 1: Optional fast3D algorithm. \n"
printf " Expected result: With low SD the mid profile is close to 2D filtering. \n\n"

rm -f cube_filtered3d.v cube_filtered2d.v

$PROGRAM -method=fast3d -steps=20 cube.v cube_filtered3d.v 3
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
$PROGRAM -method=fast2d cube.v cube_filtered2d.v 3
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=50,50,50 cube_filtered3d.v profile3dx.dat profile3dy.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgprofi -pxl=50,50,50 cube_filtered2d.v profile2dx.dat profile2dy.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacren profile3dx.dat 1 3D_3pxl_
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
tacren profile2dx.dat 1 2D_3pxl_
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
tac2svg -legend=y -mt="2D vs 3D profiles" cube_filtered_profiles_2D_vs_3D.svg -l profile3dx.dat profile2dx.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

tacmatch -abs=1.5 -rel=1 -x=y -y=y -tacnames=n -xunit=y -yunit=y profile3dx.dat profile2dx.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"







printf "\n 4.0.0 \n"
printf " Test case 1: Optional fast1D algorithm. \n"
printf " Test case 2: Set SD to zero. \n"
printf " Expected result 1: Warning. \n"
printf " Expected result 2: Output image is the same as input image. \n\n"

rm -f cube_filtered.v stdout.txt

$PROGRAM -method=fast1d cube.v cube_filtered.v 0 &> stdout.txt
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
if [ ! -f cube_filtered.v ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Warning" "with given SD no filtering is applied."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgmatch cube.v cube_filtered.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 4.1.0 \n"
printf " Test case 1: Optional fast1D algorithm. \n"
printf " Test case 2: Reasonable SD and steps. \n"
printf " Test case 3: In test image all planes are similar. \n"
printf " Expected result: Output image is the same as input image. \n\n"

rm -f output.v

$PROGRAM -method=fast1d dynamic3d_tiny.v output.v 4
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
if [ ! -f output.v ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgmatch -abs=0.1 -rel=0.1 dynamic3d_tiny.v output.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 4.2.0 \n"
printf " Test case 1: Optional fast1D algorithm. \n"
printf " Test case 2: Reasonable SD and steps. \n"
printf " Expected result: Image is smoothed only in z dimensions. \n\n"

rm -f cube_zfilt.v

$PROGRAM -method=fast1d cube.v cube_zfilt.v 4
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s cube_zfilt.v
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
imgmatch -abs=0.1 -rel=0.1 cube.v cube_zfilt.v
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgprofi -pxl=50,50,50 cube_zfilt.v profilex.dat profiley.dat profilez.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacren profilex.dat 1 X__
tacren profiley.dat 1 Y__
tacren profilez.dat 1 Z__
tac2svg -legend=y -mt="Profiles of filtered cube" cube_zfilt_profiles.svg -l profilex.dat profiley.dat profilez.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
tacmatch -abs=0.1 -rel=0.1 -x=y -y=y -tacnames=n -xunit=y -yunit=y profilex.dat profiley.dat
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
tacmatch -abs=1 -rel=5 -x=y -y=y -tacnames=n -xunit=y -yunit=y profilex.dat profilez.dat
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"






printf "\n 5.0.0 \n"
printf " Test case: ECAT 6.3 format. \n"
printf " Expected result: Same as with ECAT 7. \n\n"

rm -f filtgecat63.img
$PROGRAM dynamic.v dynamic_filtered.v 10
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

$PROGRAM dynamic.img filtgecat63.img 10
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgmatch -abs=0.1 -rel=0.1 dynamic_filtered.v filtgecat63.img
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 5.1.0 \n"
printf " Test case: Analyze format. \n"
printf " Expected result: Same as with ECAT 7. \n\n"

rm -f ana/filtgana*

$PROGRAM ana/dynamic ana/filtgana 10
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgmatch -abs=0.2 -rel=0.2 dynamic_filtered.v ana/filtgana
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 5.2.0 \n"
printf " Test case: NIfTI format. \n"
printf " Expected result: Same as with ECAT 7. \n\n"

rm -f nii/filtgnii*

$PROGRAM nii/dynamic nii/filtgnii 10
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"

imgmatch -abs=0.2 -rel=0.2 dynamic_filtered.v nii/filtgnii
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 5.3.0 \n"
printf " Test case: Static ECAT 7 format. \n"
printf " Expected result: No error. \n\n"

rm -f static_filtered.v

$PROGRAM static.v static_filtered.v 5pxl
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
img2tif -rb -s static_filtered.v static_filtered.tif
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"




printf "\n===================================================================\n"




printf "\n 6.0.0 \n"
printf " Test case: Invalid option. \n"
printf " Expected result: Error. \n\n"

rm -f stdout.txt output.v
$PROGRAM -stupidoption static.v output.v 5 &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt Error "invalid option '-stupidoption'."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"

printf "\n 6.0.1 \n"
printf " Test case: Extra argument. \n"
printf " Expected result: Error. \n\n"

rm -f stdout.txt
$PROGRAM static.v output.v 5 extra &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt Error "too many arguments."
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"

printf "\n 6.0.2 \n"
printf " Test case: Missing argument. \n"
printf " Expected result: Error. \n\n"

rm -f stdout.txt
$PROGRAM  output.v 5 &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Error"
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"

rm -f stdout.txt
$PROGRAM static.v  0  &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Error"
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"

rm -f stdout.txt
$PROGRAM static.v output.v &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Error"
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"


printf "\n 6.1.0 \n"
printf " Test case: Input file does not exist. \n"
printf " Expected result: Error. \n\n"

rm -f stdout.txt
$PROGRAM nonexistingfile.v output.v 5 &> stdout.txt
if [ $? -eq 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n ok \n"
iftisval stdout.txt "Error"
if [ $? -ne 0 ] ; then printf "Failed!\n" ; exit 1 ; fi
printf "\n passed. \n\n"




printf "\n===================================================================\n"
printf "   All passed! \n"
printf "===================================================================\n"
exit 0


