/******************************************************************************

  Copyright (c) 2006 by Turku PET Centre

  Program:     algtimes

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA 02111-1307 USA.

  Turku PET Centre hereby disclaims all copyright interest in the program.

  Juhani Knuuti
  Director, Professor
  Turku PET Centre, Turku, Finland, http://www.turkupetcentre.fi/

  Modification history:
  0.1.0 2006-07-21 Vesa Oikonen
      First created.
  0.1.1 2006-08-02 VO
      Correction in user info text.




******************************************************************************/

/*****************************************************************************/
#define PROGRAM "algtimes 0.1.1  (c) 2006 by Turku PET Centre"
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
/*****************************************************************************/
#include "libtpcmiscv.h"
#include "libtpccurveiov.h"
//#include "libtpcimgiov.h"
#include "dft.h"
#include "ift.h"
//#include "halflife.h"
/*****************************************************************************/
/* Local functions */
void print_usage();
void print_build();
/*****************************************************************************/
int TEST=0;
int SILENT=0;
extern int IFT_SPACE_BEFORE_EQ_SIGN;
/*****************************************************************************/

/*****************************************************************************/
/*
 *  main()
 */
int main(int argc, char *argv[])
{
  int ai, ii, fi, n, ret;
  char *cptr, dfile[FILENAME_MAX], ofile[FILENAME_MAX], isotope_name[7];
  char keystr[64], valuestr[64], start_time[256];
  IFT ift;
  int data_time[3], first_time[3], last_time[3], new_time[3]; // hh, mm, ss
  int protocol_duration, protocol_interval;
  int time_index=-1; // item index that contains the start time
  double count_time, channel_x, channel_y;
  double first_sample_time=0.0, last_sample_time=0.0, time_in_sec;
//double halflife=0.0, lambda, dc;


  /*
   *  Get arguments
   */
  if(argc==1) {print_usage(); return(1);}
  dfile[0]=ofile[0]=start_time[0]=isotope_name[0]=(char)0;
  for(n=0; n<3; n++) data_time[n]=first_time[n]=last_time[n]=new_time[n]=-1;
  /* Get options first */
  for(ai=1; ai<argc; ai++) if(*argv[ai]=='-') {
    cptr=argv[ai]+1; if(*cptr=='-') cptr++;
    if(strncasecmp(cptr, "TEST", 4)==0) {
      cptr+=4; if(*cptr=='=') cptr++; TEST=atoi(cptr); if(TEST<1) TEST=1;
      continue;
    } else if(strcasecmp(cptr, "V")==0 || strcasecmp(cptr, "VERSION")==0 ||
              strcasecmp(cptr, "BUILD")==0) {
      print_build();
      return(0);
    } else if(strcasecmp(cptr, "H")==0 || strcasecmp(cptr, "HELP")==0) {
      print_usage(); return(0);
    } else if(strcasecmp(cptr, "SILENT")==0) {
      SILENT=1; continue;
    } else if(strncasecmp(cptr, "o=", 2)==0) {
      cptr+=2; if(strlen(cptr)>0) {strcpy(ofile, cptr); continue;}
#if(0)
    } else if(strncasecmp(cptr, "i=", 2)==0) {
      cptr+=2; if(strlen(cptr)>0) {
        strcpy(isotope_name, cptr); cptr=hlCorrectIsotopeCode(isotope_name);
        if(cptr!=NULL) {
          strcpy(isotope_name, cptr);
          halflife=60.0*hlFromIsotope(isotope_name);
          if(halflife>0.0) continue;
        }
      }
      fprintf(stderr, "Error: invalid option for isotope '%s'\n", argv[ai]); return(1);
#endif
    }
    fprintf(stderr, "Error: invalid option '%s'.\n", argv[ai]);
    return(1);
  }
  /* Get arguments */
  for(ai=1; ai<argc; ai++) if(*argv[ai]!='-') {
    if(!dfile[0]) {strcpy(dfile, argv[ai]); continue;}
    if(!start_time[0]) {strcpy(start_time, argv[ai]); continue;}
    fprintf(stderr, "Error: invalid argument %s\n", argv[ai]);
    print_usage(); return(1);
  }
  if(!ofile[0]) strcpy(ofile, dfile);
  /* Did we get all the information that we need? */
  if(!dfile[0]) {print_usage(); return(1);}
  if(TEST) printf("************************ Test run *********************\n");
  if(TEST) {
    printf("dfile := %s\n", dfile);
    printf("ofile := %s\n", ofile);
    //printf("isotope_name := %s\n", isotope_name);
    //printf("halflife := %g\n", halflife);
  }


  /*
   *  Read the new sample start time string, if specified
   */
  if(start_time[0]) {
    n=sscanf(start_time, "%d:%d:%d", &new_time[0], &new_time[1], &new_time[2]);
    if(n<3) {
      fprintf(stderr, "Error: invalid time argument %s\n", start_time);
      return(1);
    }
    if(TEST) printf("new_time := %02d:%02d:%02d\n", new_time[0], new_time[1], new_time[2]);
    if(new_time[0]<0 || new_time[0]>23 ||
       new_time[1]<0 || new_time[1]>59 ||
       new_time[2]<0 || new_time[2]>59) {
      fprintf(stderr, "Error: invalid time argument %s\n", start_time);
      return(1);
    }
  }


  /*
   *  Read Allogg file as IFT
   */
  if(TEST) printf("reading %s\n", dfile);
  iftInit(&ift);
  ret=iftRead(&ift, dfile, 0);
  if(ret) {
    fprintf(stderr, "Error in reading '%s': %s\n", dfile, ift.status);
    return(3);
  }
  if(TEST>9) iftWrite(&ift, "stdout");
  /* Get the sampling start time */
  strcpy(start_time, "");
  for(ii=0; ii<ift.keyNr; ii++) {
    if(ift.item[ii].type!=' ' && ift.item[ii].type!=(char)0) continue;
    if(ift.item[ii].value==NULL || !ift.item[ii].value[0]) continue;
    if(TEST==19) printf("ii := %d ; value := %s\n", ii, ift.item[ii].value);
    n=strlen(ift.item[ii].value); if(n!=6) continue;
    strcpy(start_time, ift.item[ii].value);
    time_index=ii;
  }
  if(!start_time[0]) {
    fprintf(stderr, "Error: '%s' does not contain sampling start time.\n", dfile);
    iftEmpty(&ift); return(4);
  }
  n=sscanf(start_time, "%02d%02d%02d", &data_time[0], &data_time[1], &data_time[2]);
  if(n<3) {
    fprintf(stderr, "Error: invalid time argument %s in %s\n", start_time, dfile);
    iftEmpty(&ift); return(4);
  }
  if(TEST) printf("data_time := %02d:%02d:%02d\n", data_time[0], data_time[1], data_time[2]);
  if(data_time[0]<0 || data_time[0]>23 ||
     data_time[1]<0 || data_time[1]>59 ||
     data_time[2]<0 || data_time[2]>59) {
    fprintf(stderr, "Error: invalid time argument %s in %s\n", start_time, dfile);
    iftEmpty(&ift); return(4);
  }
  if(new_time[0]<0 && SILENT==0)
    fprintf(stdout, "start_time := %02d:%02d:%02d\n",
      data_time[0], data_time[1], data_time[2]);
  /* Read the protocol */
  strcpy(keystr, "Protocol"); ii=iftGet(&ift, keystr);
  if(ii<0) {
    fprintf(stderr, "Error: protocol not defined in %s.\n", dfile);
    iftEmpty(&ift); return(4);
  }
  if(TEST>8) printf("Protocol := '%s'\n", ift.item[ii].value);
  cptr=strchr(ift.item[ii].value, '('); if(cptr==NULL) cptr=ift.item[ii].value;
  n=sscanf(cptr, "%d [s] %d [ms]", &protocol_duration, &protocol_interval);
  if(n<2) {
    fprintf(stderr, "Error: invalid protocol in %s\n", dfile);
    iftEmpty(&ift); return(4);
  }
  if(TEST) printf("protocol_duration := %d\n", protocol_duration);
  if(TEST) printf("protocol_interval := %d\n", protocol_interval);
  if(new_time[0]<0 && SILENT==0)
    fprintf(stdout, "sample_interval := %g [s]\n",
      0.001*(double)protocol_interval);
  /* Read the measurement data */
  for(fi=0, ii=time_index+1; ii<ift.keyNr; ii++) {
    if(ift.item[ii].type!=' ' && ift.item[ii].type!=(char)0) continue;
    if(ift.item[ii].key!=NULL && ift.item[ii].key[0]) continue;
    if(ift.item[ii].value==NULL || !ift.item[ii].value[0]) continue;
    n=sscanf(ift.item[ii].value, "%lf %lf %lf", &count_time, &channel_x, &channel_y);
    if(n!=3) continue;
    if(TEST==18) printf("ii := %d ; value := %s\n", ii, ift.item[ii].value);
    if(fi==0) first_sample_time=count_time;
    else last_sample_time=count_time;
    fi++;
  }
  if(new_time[0]<0 && SILENT==0) fprintf(stdout, "sample_nr := %d\n", fi);
  /* Print the first sample time */
  time_in_sec=3600*data_time[0]+60*data_time[1]+data_time[2]+first_sample_time;
  first_time[0]=((int)time_in_sec)%86400;
  first_time[1]=first_time[0]%3600; first_time[0]-=first_time[1]; first_time[0]/=3600;
  first_time[2]=first_time[1]%60; first_time[1]-=first_time[2]; first_time[1]/=60;
  if(new_time[0]<0 && SILENT==0)
    fprintf(stdout, "first_sample_time := %02d:%02d:%02d\n",
      first_time[0], first_time[1], first_time[2]);
  /* Print the last sample time */
  time_in_sec=3600*data_time[0]+60*data_time[1]+data_time[2]+last_sample_time;
  last_time[0]=((int)time_in_sec)%86400;
  last_time[1]=last_time[0]%3600; last_time[0]-=last_time[1]; last_time[0]/=3600;
  last_time[2]=last_time[1]%60; last_time[1]-=last_time[2]; last_time[1]/=60;
  if(new_time[0]<0 && SILENT==0)
    fprintf(stdout, "last_sample_time := %02d:%02d:%02d\n",
      last_time[0], last_time[1], last_time[2]);
  /* Print the sampling time */
  if(new_time[0]<0 && SILENT==0)
    fprintf(stdout, "sampling_time := %d [s]\n",
     (int)(0.001*(double)protocol_interval+last_sample_time-first_sample_time));


  /*
   *  If new start time was not specified, then stop here
   */
  if(new_time[0]<0) {
    iftEmpty(&ift);  return(0);
  }

  /*
   *  If new start time is not different to the current one, then stop here
   */
  time_in_sec=3600*new_time[0]+60*new_time[1]+new_time[2]-
    (3600*data_time[0]+60*data_time[1]+data_time[2]);
  if(time_in_sec==0.0) {
    fprintf(stdout, "Start times are not different; file not changed.\n");
    iftEmpty(&ift); return(0);
  }
  if(TEST) printf("time_difference := %g\n", time_in_sec);


#if(0)
  /*
   *  Calculate the lambda
   */
  lambda=0.0; dc=1.0;
  if(halflife>0.0) {
    lambda=hl2lambda(halflife); if(time_in_sec>0) lambda=-lambda;
    if(TEST) printf("lambda := %e\n", lambda);
    dc=lambda2factor(lambda, fabs(time_in_sec), 0.0);
    if(TEST) printf("dc := %e\n", dc);
  }
#endif


  /*
   *  Change the start time
   */
  sprintf(valuestr, "%02d%02d%02d", new_time[0], new_time[1], new_time[2]);
  ret=iftReplaceNthValue(&ift, time_index, valuestr);
  if(TEST>1) iftWriteItem(&ift, time_index, stdout);


  /*
   *  Change the sample times
   */
  for(fi=0, ii=time_index+1; ii<ift.keyNr; ii++) {
    if(ift.item[ii].type!=' ' && ift.item[ii].type!=(char)0) continue;
    if(ift.item[ii].key!=NULL && ift.item[ii].key[0]) continue;
    if(ift.item[ii].value==NULL || !ift.item[ii].value[0]) continue;
    n=sscanf(ift.item[ii].value, "%lf %lf %lf", &count_time, &channel_x, &channel_y);
    if(n!=3) continue;
    if(TEST==17) printf("ii := %d ; value := %s\n", ii, ift.item[ii].value);
    count_time -= time_in_sec;
    sprintf(valuestr, "%4.1f %5.0f %5.0f", count_time, channel_x, channel_y);
    ret=iftReplaceNthValue(&ift, ii, valuestr);
    fi++;
  }


  /*
   *  Write the corrected Allogg file
   */
  if(TEST) printf("saving %s\n", ofile);
  IFT_SPACE_BEFORE_EQ_SIGN=0;
  ret=iftWrite(&ift, ofile);
  if(ret) {
    fprintf(stderr, "Error in writing '%s': %s\n", ofile, ift.status);
    iftEmpty(&ift); return(11);
  }


  /* free memory */
  iftEmpty(&ift);

  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
/**
 *  void print_usage()
 */
void print_usage()
{
  int i=0;
  static char *info[] = {
  " ",
  PROGRAM,
  " ",
  "Correct the sample times in Allogg ABSS on-line blood sampler data.",
  "Use this, if sampling was accidentally started later than the PET scan.",
  "If the new sampling start time is not given, the program lists information",
  "on current sample times on the screen.",
  " ",
  "Usage:",
  "  algtimes [Options] <Sampler datafile> [New sampling start time as hh:mm:ss]",
  " ",
  "Options:",
  " -o=<Filename>",
  "    Filename for corrected sampler data; by default, original file is",
  "    overwritten.",
//" -i=<isotope code>",
//"    By default, only sample times are changed. To correct the radioactivities,",
//"    you must specify the tracer isotope, for example O-15, N-13, C-11, F-18,",
//"    Ge-68, Ga-68, Br-76, Rb-82, or Cu-62 with option -i.",
  " -v[ersion]",
  " -build",
  "    Print build information and exit.",
  " -h[elp]",
  "    Print this text and exit.",
  " -silent",
  "    Work silently; only warnings and error messages are printed in stdout.",
  " ",
  "See also: blo2kbq, injdifft, blotimes",
  " ",
  "Keywords: input, blood, sample times, injection time",
  " ",
  0};
  while(info[i]!=0) printf("  %s\n", info[i++]);
}
/*****************************************************************************/

/*****************************************************************************/
/**
 * Prints build information to stdout.
 */
void print_build() {

  printf("\n %s\n", PROGRAM);
  printf("\n Build %s %s\n\n",__DATE__,__TIME__);
#if defined(__FILE__) && defined(__TIMESTAMP__)
  printf(" File %s last edited on %s\n", __FILE__, __TIMESTAMP__);
#endif
  printf(" _______________________________________________\n");

//libtpcimgio_print_build(stdout);
//printf(" _______________________________________________\n");
//  libtpcmodel_print_build(stdout);
//  printf(" _______________________________________________\n");
  libtpccurveio_print_build(stdout);
  printf(" _______________________________________________\n");
  libtpcmisc_print_build(stdout);
  printf(" _______________________________________________\n");

#if defined(__STDC_VERSION__)
  printf(" Version of C: %ld\n", __STDC_VERSION__);
  printf(" _______________________________________________\n\n");
#endif
#if defined(__STDC_ISO_10646__)
  printf(" Compiler ISO/IEC 10646 specification: %ld\n", __STDC_ISO_10646__);
  printf(" _______________________________________________\n\n");
#endif
#if defined(__GNUC__) && defined(__VERSION__)
  printf(" GNU C version: %s\n", __VERSION__);
  printf(" _______________________________________________\n\n");
#endif
}
/*****************************************************************************/

/*****************************************************************************/
