/******************************************************************************

  Copyright (c) 2006-2009 Turku PET Centre

  Program:     blotimes
  Description: List or change sample times in blood online sampler file.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.

  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA 02111-1307 USA.

  Turku PET Centre hereby disclaims all copyright interest in the program.
  Juhani Knuuti
  Director, Professor
  Turku PET Centre, Turku, Finland, http://www.turkupetcentre.fi/

  Modification history:
  1.0.0 2006-06-26 Vesa Oikonen
        First created.
  1.0.1 2008-11-07 VO
        Tiny updates which do not affect results.
        Options --version, --build, --help, --verbose, --silent.
        Backup file named as .bak instead of percentage.
  1.0.2 2008-12-04 VO
        Allows more than 5 comment lines.
  1.0.3 2008-12-14 VO
        Some functionality moved into separate c file.
  1.1.0 2009-03-16 VO
        Lots of changes in scanditronics.c.
        Start time in header is changed also when sample times are changed.
        Lots of chages in user info text.
         



******************************************************************************/
#define PROGRAM "blotimes 1.1.0 (c) 2006-2009 by Turku PET Centre"
/*****************************************************************************/
#define MAX_LINE_LEN 1024
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
/*****************************************************************************/
#include "scanditronics.h"
/*****************************************************************************/
void print_usage();
void print_build();
/*****************************************************************************/
int TEST=0;
int SILENT=0;
int VERBOSE=0;
/*****************************************************************************/


/*****************************************************************************/
/*
 *   main()
 */
int main(int argc, char *argv[])
{
  int ai, fi, n, ret;
  char *cptr, dfile[FILENAME_MAX], temp[1024];
  BData bdata;
  char start_time[256];
  int data_time[3], new_time[3]; /* hh, mm, ss */
  int dur;
  struct tm *tm_meas;



  /*
   *  Get and check arguments
   */
  if(argc==1) {print_usage(); return(1);}
  dfile[0]=start_time[0]=(char)0;
  /* Get options first */
  for(ai=1; ai<argc; ai++) if(*argv[ai]=='-') {
    cptr=argv[ai]+1; if(*cptr=='-') cptr++; if(cptr==NULL) continue;
    if(strncasecmp(cptr, "TEST", 4)==0) {
      cptr+=4; if(*cptr=='=') cptr++; TEST=atoi(cptr); if(TEST<1) TEST=1;
      continue;
    } else if(strcasecmp(cptr, "V")==0 || strcasecmp(cptr, "VERSION")==0 ||
              strcasecmp(cptr, "BUILD")==0) {
      print_build();
      return(0);
    } else if(strcasecmp(cptr, "H")==0 || strcasecmp(cptr, "HELP")==0) {
      print_usage(); return(0);
    } else if(strcasecmp(cptr, "SILENT")==0) {
      SILENT=1; VERBOSE=0; continue;
    } else if(strcasecmp(cptr, "VERBOSE")==0) {
      VERBOSE=1; SILENT=0; continue;
    }
    fprintf(stderr, "Error: invalid option '%s'\n", argv[ai]); return(1);
  }
  /* Get arguments */
  for(ai=1; ai<argc; ai++) if(*argv[ai]!='-') {
    if(!dfile[0]) {strcpy(dfile, argv[ai]); continue;}
    if(!start_time[0]) {strcpy(start_time, argv[ai]); continue;}
    fprintf(stderr, "Error: invalid argument %s\n", argv[ai]);
    print_usage(); return(1);
  }
  /* Did we get all the information that we need? */
  if(!dfile[0]) {
    fprintf(stderr, "Error: missing command-line argument; try %s --help\n",
      argv[0]);
    return(1);
  }
  if(TEST) {
    printf("TEST := %d\n", TEST);
    printf("SILENT := %d\n", SILENT);
    printf("VERBOSE := %d\n", VERBOSE);
    printf("dfile := %s\n", dfile);
    printf("start_time := %s\n", start_time);
  }
  if(TEST>0) VERBOSE=1;


  /*
   *  Read the new sample start time string, if specified
   */
  if(start_time[0]) {
    n=sscanf(start_time, "%d:%d:%d", &new_time[0], &new_time[1], &new_time[2]);
    if(n<3) {
      fprintf(stderr, "Error: invalid time argument %s\n", start_time);
      return(1);
    }
    if(VERBOSE)
      printf("new_time := %02d:%02d:%02d\n", new_time[0], new_time[1], new_time[2]);
    if(new_time[0]<0 || new_time[0]>23 ||
       new_time[1]<0 || new_time[1]>59 ||
       new_time[2]<0 || new_time[2]>59) {
      fprintf(stderr, "Error: invalid time argument %s\n", start_time);
      return(1);
    }
  }


  /*
   *  Read blood activity data from file
   */
  if(VERBOSE) {printf("reading file %s\n", dfile); fflush(stdout);}
  bdata_init(&bdata);
  ret=bdata_read(dfile, &bdata, TEST, temp);
  if(ret!=0) {
    fprintf(stderr, "Error in reading %s: %s\n", dfile, temp);
    return 2;
  }

  /* Print the sample nr */
  if(!start_time[0] || TEST)
    fprintf(stdout, "sample_nr := %d\n", bdata.sample_nr);

  /* Calculate the sampling start time from the first sample */
  tm_meas=localtime(&bdata.start_time); if(tm_meas==NULL) {
    fprintf(stderr, "Error: invalid start time in %s\n", dfile);
    bdata_empty(&bdata); return 2;
  }
  tm_meas->tm_isdst=-1;
  data_time[0]=tm_meas->tm_hour;
  data_time[1]=tm_meas->tm_min;
  data_time[2]=tm_meas->tm_sec;
  if(TEST || VERBOSE) {
    strftime(temp, 128, "%Y-%m-%d", tm_meas);
    printf("sampling_start_time := %s\n", temp);
  }
  if(!start_time[0] || TEST || VERBOSE) {
    strftime(temp, 128, "%H:%M:%S", tm_meas);
    fprintf(stdout, "first_sample_time := %s\n", temp);
  }

  /* Print the sample interval */
  if(!start_time[0] || TEST) {
    fprintf(stdout, "sample_interval := %g [s]\n", bdata.sample[0].meas_interval);
  }
  /* Calculate and print the sampling duration */
  if(!start_time[0] || TEST) {
    dur =+bdata.sample[bdata.sample_nr-1].time_since_start
         -bdata.sample[0].time_since_start
         +bdata.sample[bdata.sample_nr-1].meas_interval;
    fprintf(stdout, "sampling_time := %d [s]\n", dur);
  }

  /*
   *  If new start time was not specified, then stop here
   */
  if(!start_time[0]) {bdata_empty(&bdata); return(0);}


  /*
   *  If new start time is not different to the current one, then stop here
   */
  if(data_time[0]==new_time[0] && data_time[1]==new_time[1] && data_time[2]==new_time[2]) {
    if(SILENT==0)
      fprintf(stdout, "Start times are not different; file not changed.\n");
    bdata_empty(&bdata); return(0);
  }


  /*
   *  Change the sample times
   */
  bdata.sample[0].time_of_day = 3600*new_time[0] + 60*new_time[1] + new_time[2];
  for(fi=1; fi<bdata.sample_nr; fi++)
    bdata.sample[fi].time_of_day =
      bdata.sample[fi-1].time_of_day + bdata.sample[fi-1].meas_interval;


  /*
   *  Change the scanner start time
   */
  tm_meas->tm_hour=new_time[0];
  tm_meas->tm_min=new_time[1];
  tm_meas->tm_sec=new_time[2];
  bdata.start_time=mktime(tm_meas); if(bdata.start_time==(time_t)-1) {
    fprintf(stderr, "Error: invalid start time.\n");
    bdata_empty(&bdata); return 5;
  }
  /* Correct scan start time in header too */
  ret=bdata_correct_header(&bdata); if(ret!=0) {
    fprintf(stderr, "Error: cannot correct header time.\n");
    bdata_empty(&bdata); return 6;
  }


  /*
   *  Write down the edited sampler data, and backup the original file
   */
  if(VERBOSE) printf("writing edited data in %s\n", dfile);
  strcpy(temp, dfile); strcat(temp, ".bak"); remove(temp); rename(dfile, temp);
  /* Write down the corrected sampler data */
  ret=bdata_write(&bdata, dfile, TEST, temp);
  if(ret!=0) {
    fprintf(stderr, "Error in writing %s: %s\n", dfile, temp);
    bdata_empty(&bdata); return 11;
  }

  /* Free memory */
  bdata_empty(&bdata);

  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
/*!
 *  Prints usage information to stdout.
 */
void print_usage() {
  int i=0;
  static char *info[] = {
  " ",
  PROGRAM,
  " ",
  "Correct or list the sample times in Scanditronics or GEMS on-line blood",
  "sampler (blood pump) data (1).",
  "Correct decay correction will bepossible only if blood sampling start time",
  "is correct versus PET scan start time.",
  " ",
  "Usage:  blotimes <Sampler datafile> [New sampling start time as hh:mm:ss]",
  " ",
  "Options:",
  " -h, --help",
  "     Print this message and exit",
  " --build",
  "     Print software build information and exit",
  " --silent",
  "     Program works silently, printing only warnings and error messages.",
  " --verbose",
  "     Program prints lots of information about what it is doing.",
  " ",
  "In Scanditronics data files the correct sampling start time is in the file",
  "headers; however, in old Scanditronics data files the headers do not contain",
  "time, but it is read from the first sample data line.",
  "In GEMS data files the time in file headers is not the sampling start time,",
  "but time is read from the first sample data line.",
  "If new sampling time is given and it is different than the current sampling",
  "start time, then all sample times in the sampler data file are changed.",
  "If the new sampling start time is not given, the program lists information",
  "on current sample times on the screen.",
  " ",
  "Example 1a: Blood sampler times are listed:",
  "      blotimes us1165.bld",
  "Example 1b: PET image scan start time is listed:",
  "      egetstrt us1165.v",
  "Example 1c: Obviously, blood sampler clock was not synchronized with PET,",
  "therefore blood sample times must be corrected to PET scan start time:",
  "      blotimes us1165.bld 10:25:58",
  " ",
  "References:",
  "1. http://www.turkupetcentre.net/analysis/doc/pumpdata.html",
  " ",
  "See also: algtimes, blo2kbq, injdifft, egetstrt, blozero, blofch",
  " ",
  "Keywords: blood, input, sample times, Scanditronics, online-sampler",
  " ",
  "This program comes with ABSOLUTELY NO WARRANTY. This is free software, and",
  "you are welcome to redistribute it under GNU General Public License.",  
  " ",
  0};
  while(info[i]!=0) printf("  %s\n", info[i++]);
}
/*****************************************************************************/

/*****************************************************************************/
/*!
 * Prints build information to stdout.
 */
void print_build(){

  printf("\n %s\n", PROGRAM);
  printf("\n Build %s %s\n\n",__DATE__,__TIME__);
#if defined(__FILE__) && defined(__TIMESTAMP__)
  printf(" File %s last edited on %s\n", __FILE__, __TIMESTAMP__);
#endif
  printf(" _______________________________________________\n");

  //libtpcmisc_print_build(stdout);
  //printf(" _______________________________________________\n");

#if defined(__STDC_VERSION__)
  printf(" Version of C: %ld\n", __STDC_VERSION__);
  printf(" _______________________________________________\n\n");
#endif
#if defined(__STDC_ISO_10646__)
  printf(" Compiler ISO/IEC 10646 specification: %ld\n", __STDC_ISO_10646__);
  printf(" _______________________________________________\n\n");
#endif
#if defined(__GNUC__) && defined(__VERSION__)
  printf(" GNU C version: %s\n", __VERSION__);
  printf(" _______________________________________________\n\n");
#endif
#if defined(__linux__)
  printf(" OS: linux\n");
#elif defined(__unix__)
  printf(" OS: unix\n");
#elif defined(__WIN32__) || defined(__WINNT__)
  printf(" OS: Windows\n");
#endif
#if defined(__sun__) || defined(__sparc__)
  printf(" Platform: sun\n");
  printf(" _______________________________________________\n\n");
#elif defined(__i486__) || defined(__i386__) || defined(_X86_)
  printf(" Platform: x86\n");
  printf(" _______________________________________________\n\n");
#endif
}
/*****************************************************************************/

/*****************************************************************************/
