/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpccsv.h"
/*****************************************************************************/

/*****************************************************************************/
int test_csvInit(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  csvInit(&csv);
  csvInit((CSV*)NULL);
  
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvFree(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  csvInit(&csv);
  csvFree(&csv);
  
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvAllocate(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  int n, ret;
  csvInit(&csv);

  n=0;
  ret=csvAllocate(&csv, n);
  if(ret!=TPCERROR_OK || csv._item_nr!=0) {csvFree(&csv); return 1;}
  
  n=10;
  ret=csvAllocate(&csv, n);
  if(ret!=TPCERROR_OK || csv._item_nr!=10) {csvFree(&csv); return 2;}
  
  n=5;
  ret=csvAllocate(&csv, n);
  if(ret!=TPCERROR_OK || csv._item_nr!=10) {csvFree(&csv); return 3;}
  
  n=15;
  ret=csvAllocate(&csv, n);
  if(ret!=TPCERROR_OK || csv._item_nr!=15) {csvFree(&csv); return 4;}
  
  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvPutString(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  int ret;
  char str[256];

  csvInit(&csv);

  strcpy(str, "eka"); if(verbose>1) printf("'%s' on line 1\n", str);
  ret=csvPutString(&csv, str, 0);
  if(verbose>2)
    printf("  -> ret=%d nr=%d space=%d\n", ret, csv.nr, csv._item_nr);
  if(ret!=TPCERROR_OK || csv.nr!=1 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 1;
  }
  
  strcpy(str, "toka"); printf("'%s' on line 1\n", str);
  ret=csvPutString(&csv, str, 0);
  if(ret!=TPCERROR_OK || csv.nr!=2 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 2;
  }
  
  strcpy(str, ""); printf("'%s' on line 1\n", str);
  ret=csvPutString(&csv, str, 0);
  if(ret!=TPCERROR_OK || csv.nr!=3 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 3;
  }
  
  strcpy(str, ""); printf("'%s' on line 1\n", "NULL");
  ret=csvPutString(&csv, NULL, 0);
  if(ret!=TPCERROR_OK || csv.nr!=4 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 4;
  }
  
  strcpy(str, "2nd line"); printf("'%s' on line 2\n", str);
  ret=csvPutString(&csv, str, 1);
  if(ret!=TPCERROR_OK || csv.nr!=5 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 5;
  }
  
  strcpy(str, " "); printf("'%s' on line 2\n", str);
  ret=csvPutString(&csv, str, 0);
  if(ret!=TPCERROR_OK || csv.nr!=6 || strcmp(csv.c[csv.nr-1].content, str)!=0) {
    csvFree(&csv); return 6;
  }

  if(verbose>2) printf("  row_nr=%d\n  col_nr=%d\n", csv.row_nr, csv.col_nr);
  if(csv.row_nr!=2 || csv.col_nr!=4) {csvFree(&csv); return 10;}  

  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvPutDouble(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  int ret;
  double v;

  csvInit(&csv);
  csv.separator=',';

  v=1.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=1.0E-03; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(verbose>2) printf("  row_nr=%d\n  col_nr=%d\n", csv.row_nr, csv.col_nr);
  if(csv.row_nr!=2 || csv.col_nr!=4) {csvFree(&csv); return 10;}  

  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvPutInt(
  int verbose
) {
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  CSV csv;
  int ret;
  int v;

  csvInit(&csv);
  csv.separator=',';

  v=1; ret=csvPutInt(&csv, v, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2; ret=csvPutInt(&csv, v, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=-3; ret=csvPutInt(&csv, v, 1);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(verbose>2) printf("  row_nr=%d\n  col_nr=%d\n", csv.row_nr, csv.col_nr);
  if(csv.row_nr!=2 || csv.col_nr!=2) {csvFree(&csv); return 10;}
  if(strcmp(csv.c[0].content, "1")!=0) {csvFree(&csv); return 11;} 
  if(strcmp(csv.c[1].content, "2")!=0) {csvFree(&csv); return 12;} 
  if(strcmp(csv.c[2].content, "-3")!=0) {csvFree(&csv); return 13;} 

  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvRowLength(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  int n, r, ret;
  CSV csv; csvInit(&csv);
  
  /* just checking that does not crash */
  n=csvRowLength(NULL, 0);
  if(n!=0) return(1);
  n=csvRowLength(&csv, 0);
  if(n!=0) return(2);
  
  csv.separator=',';
  double v;
  v=1.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=1.0E-03; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  r=0; n=csvRowLength(&csv, r); if(n!=4) {csvFree(&csv); return(11);}
  r=1; n=csvRowLength(&csv, r); if(n!=3) {csvFree(&csv); return(12);}
  r=2; n=csvRowLength(&csv, r); if(n!=0) {csvFree(&csv); return(13);}

  if(verbose>2) printf("  ok\n");
  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvIsRegular(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  int ret;
  CSV csv; csvInit(&csv);
  
  /* just checking that does not crash */
  ret=csvIsRegular(NULL);
  if(ret!=0) return(1);
  ret=csvIsRegular(&csv);
  if(ret!=1) return(2);
  
  csv.separator=',';
  double v;
  v=1.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=1.0E-03; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(csvIsRegular(&csv)) {csvFree(&csv); return(11);}

  v=4.0E-03; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(!csvIsRegular(&csv)) {csvFree(&csv); return(12);}

  csvFree(&csv);

  v=1.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 1, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(!csvIsRegular(&csv)) {csvFree(&csv); return(13);}


  csvFree(&csv);

  v=1.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=2.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}
  v=3.0; ret=csvPutDouble(&csv, v, 0, 0);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return 1;}

  if(!csvIsRegular(&csv)) {csvFree(&csv); return(14);}

  csvFree(&csv);

  if(verbose>2) printf("  ok\n");
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvCell(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  CSV csv; csvInit(&csv);
  char *cptr;
  
  /* just checking that does not crash */
  cptr=csvCell(NULL, 0, 0);
  if(cptr!=NULL) return(1);
  cptr=csvCell(&csv, 0, 0);
  if(cptr!=NULL) return(2);
  
  csv.separator=',';

  /* Fill CVS with strings */
  char temp[64];
  int ret, ci, ri, colnr=6, rownr=4;  
  for(ri=0; ri<rownr; ri++) {
    for(ci=0; ci<colnr; ci++) {
      sprintf(temp, "row%dcol%d", ri+1, ci+1);
      if(ri>0 && ci==0) ret=csvPutString(&csv, temp, 1);
      else ret=csvPutString(&csv, temp, 0);
      if(ret!=TPCERROR_OK) break;
    }
  }

  ci=0; ri=0;
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  %s\n", cptr);
  if(cptr==NULL || strcmp(cptr, "row1col1")) {csvFree(&csv); return(11);}

  ci=3; ri=2;
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  %s\n", cptr);
  if(cptr==NULL || strcmp(cptr, "row3col4")) {csvFree(&csv); return(12);}

  ci=30; ri=20;
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(cptr!=NULL) {csvFree(&csv); return(13);}
  if(verbose>1) printf("  NULL\n");
  
  if(verbose>2) printf("  ok\n");
  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
int test_csvCellReplace(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }
  
  CSV csv; csvInit(&csv);
  int ret=0;
  
  if(verbose>1) printf("\nTest with stupid input\n");
  ret=csvCellReplace(NULL, 0, 0, NULL);
  if(ret!=TPCERROR_FAIL) return(1);
  ret=csvCellReplace(&csv, 0, 0, NULL);
  if(ret==TPCERROR_OK) return(2);
  
  csv.separator=',';

  /* Fill CVS with strings */
  char temp[64];
  int ci, ri, colnr=6, rownr=4;  
  for(ri=0; ri<rownr; ri++) {
    for(ci=0; ci<colnr; ci++) {
      sprintf(temp, "row%dcol%d", ri+1, ci+1);
      if(ri>0 && ci==0) ret=csvPutString(&csv, temp, 1);
      else ret=csvPutString(&csv, temp, 0);
      if(ret!=TPCERROR_OK) break;
    }
  }

  if(verbose>1) printf("\nTest valid replacements\n");
  char *cptr;

  ci=0; ri=0;
  if(verbose>2) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  %s\n", cptr);
  if(verbose>2) printf("replace it with 'meh'\n");
  ret=csvCellReplace(&csv, ri, ci, "meh");
  if(ret!=TPCERROR_OK) {csvFree(&csv); return(11);}
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  -> %s\n", cptr);
  if(cptr==NULL || strcmp(cptr, "meh")) {csvFree(&csv); return(12);}


  ci=3; ri=2;
  if(verbose>2) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  %s\n", cptr);
  if(verbose>2) printf("set it to empty value\n");
  ret=csvCellReplace(&csv, ri, ci, NULL);
  if(ret!=TPCERROR_OK) {csvFree(&csv); return(21);}
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  -> %s\n", cptr);
  if(cptr==NULL || strlen(cptr)>0) {csvFree(&csv); return(22);}


  ci=5; ri=3;
  if(verbose>2) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  %s\n", cptr);
  if(verbose>2) printf("set it to empty value\n");
  ret=csvCellReplace(&csv, ri, ci, "");
  if(ret!=TPCERROR_OK) {csvFree(&csv); return(31);}
  if(verbose>1) printf("reading cell[%d][%d]:\n", ci, ri);
  cptr=csvCell(&csv, ri, ci);
  if(verbose>1) printf("  -> %s\n", cptr);
  if(cptr==NULL || strlen(cptr)>0) {csvFree(&csv); return(32);}


  if(verbose>1) printf("\nTest with cell that does not exist\n");
  ci=6; ri=4;
  ret=csvCellReplace(&csv, ri, ci, "hehe");
  if(ret!=TPCERROR_MISSING_DATA) {csvFree(&csv); return(41);}
  
  if(verbose>1) printf("  ok\n");
  csvFree(&csv);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
