/******************************************************************************
 * This file is not compiled into the library, but it contains main()
 * which is compiled to an executable, used to test the library functions. 
 *****************************************************************************/

/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpcextensions.h"
/*****************************************************************************/

/*****************************************************************************/
static char *info[] = {
  "Usage: @P [options]",
  " ",
  "Options:",
  " -stdoptions", // List standard options like --help, -v, etc
  " -t, --test",
  "     Run all tests for library functions.",
  0};
/*****************************************************************************/

/*****************************************************************************/
/** Run unit tests to the library functions
 *  @author Vesa Oikonen
 *  @return 0 if all tests pass, otherwise >0.
 * */
int main(
  /** Nr of arguments */
  int argc,
  /** Pointer to arrays of argument string */
  char *argv[ ]
) {
  int i, help=0, version=0, verbose=1, error=0, test=0;
  int ret;
  char *cptr;

  if(argc==1) {tpcPrintUsage(argv[0], info, stdout); return(0);}
  for(i=1; i<argc; i++) {
    if(tpcProcessStdOptions(argv[i], &help, &version, &verbose)==0) continue;
    cptr=argv[i]; if(*cptr=='-') cptr++; if(*cptr=='-') cptr++;
    if(strncasecmp(cptr, "TEST", 1)==0) {
      test=1; continue;
    } else {
      error++; break;
    }
  }
  if(error>0) {
    fprintf(stderr, "Error: specify --help for usage.\n");
    return(1);
  }
  /* Print help or version? */
  if(help) {tpcPrintUsage(argv[0], info, stdout); return(0);}
  if(version) {tpcPrintBuild(argv[0], stdout); return(0);}

  if(test==0) return(0);

  if(verbose>0) printf("running tests for library functions...\n");
  TPCSTATUS status; statusInit(&status); status.verbose=verbose;
  statusSet(&status, __func__, __FILE__, __LINE__, 0);
  i=10;
  /* strext */
  i++; if((ret=test_strTokenNr(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strTokenNCpy(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strChrCount(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strReplaceChar(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#ifndef HAVE_STRNDUP // Test only if own implementation is used
  i++; if((ret=test_strndup(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#endif
  i++; if((ret=test_strstrNoQuotation(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strncpyCleanSpaces(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strCleanSpaces(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strncpyClean(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strClean(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strTokenDup(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strInPars(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strCleanPars(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strncatInt(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strncatDouble(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strncatIntZ(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#ifndef HAVE_STRNLEN // Test only if own implementation is used
  i++; if((ret=test_strnlen(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#endif
#ifndef HAVE_STRLCAT // Test only if own implementation is used
  i++; if((ret=test_strlcat(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#endif
#ifndef HAVE_STRLCPY // Test only if own implementation is used
  i++; if((ret=test_strlcpy(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
#endif
  i++; if((ret=test_strIsSpaceOnly(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}

  /* decpoint */
  i++; if((ret=test_strIsValidNumber(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_atofVerified(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_atofCheck(&status)!=0)) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strHaveDecimalComma(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strHaveDecimalSeparator(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_atofList(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* filename */
  i++; if((ret=test_filenameRmPath(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_filenameRmFile(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_filenameRmExtension(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_filenameRmExtensions(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_fnmatch(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_fncasematch(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_filenameGetExtension(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_filenameGetExtensions(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* proginfo */
  i++; if((ret=test_tpcProcessStdOptions(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tpcPrintUsage(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tpcPrintBuild(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tpcProgramName(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tpcYesNo(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* units */
  i++; if((ret=test_unitName(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIdentify(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIdentifyFilename(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsDistance(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsTime(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsVolume(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsMass(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsMole(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsRadioactivity(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitIsCombinatorial(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitConversionFactor(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitInverse(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitDividerHasVolume(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitDividerHasMass(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitCombination(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_unitDividerMassVolumeConversion(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}

  /* statusmsg */
  i++; if((ret=test_statusPrint(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_errorMsg(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* readasciifile */
  i++; if((ret=test_asciiFileSize(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_asciiFileRead(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_asciiCommentLine(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* datetime */
  i++; if((ret=test_strDateValid(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateValid2(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateValid3(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateValid4(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strTimeValid(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateTimeValid(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateTimeRead(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateRead(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_time_to_tm(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tmDifference(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_tmAdd(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_strDateTimeDifference(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* studynr */
  i++; if((ret=test_studynrVerify(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_studynrStandardize(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_studynrFromFilename(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* doubleutil */
  i++; if((ret=test_doubleMatch(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleMatchRel(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleMachEps(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleCopy(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleMaxIndex(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleSum(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleMean(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleGetWithUnit(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleSpanPositives(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleCSpanPositives(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_doubleNonzeroes(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* floatutil */
  i++; if((ret=test_floatMatch(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatMatchRel(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatMachEps(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatCopy(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatMaxIndex(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatSum(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatMean(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatGetWithUnit(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatSpanPositives(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatCSpanPositives(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_floatNonzeroes(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* intutil */
  i++; if((ret=test_atoiCheck(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistInit(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistFree(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistAdd(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistSort(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistAddFromString(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_intlistExpandFromString(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* roiname */
  i++; if((ret=test_roinameSubpart(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_roinameEditByTemplate(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_roinameAddField(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_roinameExists(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_roinameMatch(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  /* endian */
  i++; if((ret=test_endianLittle(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_swap(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_swabip(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_swawbip(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}
  i++; if((ret=test_swawip(&status))!=0) {
    fprintf(stderr, "failed (%d).\n", ret); return(i);}

  if(verbose>0) printf("\nAll tests passed.\n\n");
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
