/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpcnlopt.h"
/*****************************************************************************/

/*****************************************************************************/
int test_nloptMPSO(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret, parNr;
  NLOPT nlo; nloptInit(&nlo);
  double e;

  /* Seed for random number generator */
  drandSeed(1);

  if(verbose>1) printf("\ncalling function with empty input\n");
  ret=nloptMPSO(&nlo, 0, status);
  if(ret==TPCERROR_OK) return(1);
  if(verbose>2) printf("  -> error returned, which is good\n");


  if(verbose>1) printf("\n Rosenbrock's valley with N=2 \n");
  parNr=2;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(11);}
  /* set parameter limits */
  nlo.xlower[0]=0.0;  nlo.xupper[0]=100.0;  nlo.xtol[0]=0.001;
  nlo.xlower[1]=0.1;  nlo.xupper[1]=50.0;   nlo.xtol[1]=0.001;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  nlo.maxFunCalls=50000;
  //srand(10); // to get same sequence of random numbers each time
  ret=nloptMPSO(&nlo, 1, NULL /*status*/);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(12);}
  if(verbose>2) for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    e=fabs(nlo.xfull[i]-1.0);
    if(e>5.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {nloptFree(&nlo); return(13);}
  nloptFree(&nlo);


  if(verbose>1) printf("\n Rosenbrock's valley with N=6 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=6;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(21);}
  /* set initial parameters */
  nlo.xfull[0]=0.91;
  nlo.xfull[1]=1.11;
  nlo.xfull[2]=0.95;
  nlo.xfull[3]=1.05;
  nlo.xfull[4]=0.99;
  nlo.xfull[5]=1.15;
  for(int i=0; i<6; i++) {
    nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0; 
    nlo.xtol[i]=0.001; nlo.xdelta[i]=5.0E-02;
  }
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  nlo.maxFunCalls=50000;
  ret=nloptMPSO(&nlo, 1, NULL /*status*/);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(22);}
  if(verbose>2) for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    e=fabs(nlo.xfull[i]-1.0);
    if(e>5.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {nloptFree(&nlo); return(23);}
  nloptFree(&nlo);



  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}
/*****************************************************************************/

/*****************************************************************************/
