/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include "tpcextensions.h"
#include "test_tpcnlopt.h"
/*****************************************************************************/

/*****************************************************************************/
int test_nloptSimplex(
  TPCSTATUS *status
) {
  int verbose=0; if(status!=NULL) verbose=status->verbose;
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  if(verbose>0) {
    printf("\n=====================================\n");
    printf("\n%s\n", __func__);
    printf("\n=====================================\n");
  }

  int ret, parNr;
  NLOPT nlo; nloptInit(&nlo);
  double e;

  if(verbose>1) printf("\ncalling function with empty input\n");
  ret=nloptSimplex(&nlo, 0, status);
  if(ret==TPCERROR_OK) return(1);
  if(verbose>2) printf("  -> error returned, which is good\n");

  if(verbose>1) printf("\n Simplex should return error if N=1 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=1;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(2);}
  /* set initial parameters */
  nlo.xfull[0]=0.99; 
  nlo.xlower[0]=-100.0; nlo.xupper[0]=100.0; 
  nlo.xtol[0]=1.0E-08; nlo.xdelta[0]=1.0E-02;
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptSimplex(&nlo, 0, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(ret==TPCERROR_OK) {nloptFree(&nlo); return(3);}
  nloptFree(&nlo);

  if(verbose>1) printf("\n Rosenbrock's valley with N=2 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=2;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(11);}
  /* set initial parameters */
  nlo.xfull[0]=0.9;
  nlo.xfull[1]=1.1;
  for(int i=0; i<2; i++) {
    nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0; 
    nlo.xtol[i]=1.0E-08; nlo.xdelta[i]=1.0E-02;
  }
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptSimplex(&nlo, 0, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(12);}
  if(verbose>2)
    for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    e=fabs(nlo.xfull[i]-1.0);
    if(e>1.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {nloptFree(&nlo); return(13);}
  nloptFree(&nlo);

  if(verbose>1) printf("\n Rosenbrock's valley with N=6 \n");
  statusSet(status, __func__, __FILE__, __LINE__, 0);
  parNr=6;
  ret=nloptAllocate(&nlo, parNr);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(21);}
  /* set initial parameters */
  nlo.xfull[0]=0.91;
  nlo.xfull[1]=1.11;
  nlo.xfull[2]=0.95;
  nlo.xfull[3]=1.05;
  nlo.xfull[4]=0.99;
  nlo.xfull[5]=1.15;
  for(int i=0; i<6; i++) {
    nlo.xlower[i]=-100.0; nlo.xupper[i]=100.0; 
    nlo.xtol[i]=1.0E-08; nlo.xdelta[i]=5.0E-02;
  }
  /* set object function */
  nlo._fun=nloptDejong2;
  /* Optimize */
  ret=nloptSimplex(&nlo, 0, status);
  if(verbose>2) printf("  -> ret := %d\n", ret);
  if(verbose>3) printf("  -> function_calls := %d\n", nlo.funCalls);
  if(ret!=TPCERROR_OK) {nloptFree(&nlo); return(22);}
  if(verbose>2)
    for(int i=0; i<parNr; i++) printf("  x[%d] := %g\n", i, nlo.xfull[i]);
  /* Check optimized parameters */
  ret=0;
  for(int i=0; i<parNr; i++) {
    e=fabs(nlo.xfull[i]-1.0);
    if(e>1.0E-03) {
      ret++; if(verbose>2) printf("  |x-true|=%g\n", e);
    }
  }
  if(ret>0) {nloptFree(&nlo); return(23);}
  nloptFree(&nlo);

  statusSet(status, __func__, __FILE__, __LINE__, 0);
  return(0);
}

#if(0)
  double f, par[50], delta[50];
  int i, parNr, ret, iterNr;


  printf("test_powell(): Rosenbrock's valley with N=2\n");
  parNr=2; iterNr=100;
  for(i=0; i<parNr; i++) {
    par[i]=0.3*(double)(i+1); delta[i]=0.2;
  }
  ret=powell(par, delta, parNr, 0.002, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  for(i=0; i<parNr; i++) {
    if(fabs(par[i]-1.0)>1.0E-06) {
      if(VERBOSE) {printf("   FAILED: powell() did not reach required minimum\n");}
      return (1);
    }
  }

  
  printf("test_powell(): Rosenbrock's valley with N=3\n");
  parNr=3; iterNr=100;
  for(i=0; i<parNr; i++) {
    par[i]=0.3*(double)(i+1); delta[i]=0.2;
  }
  ret=powell(par, delta, parNr, 0.002, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  for(i=0; i<parNr; i++) {
    if(fabs(par[i]-1.0)>1.0E-06) {
      if(VERBOSE) {printf("   FAILED: powell() did not reach required minimum\n");}
      return (1);
    }
  }

  
  printf("test_powell(): Rosenbrock's valley with N=6\n");
  parNr=6; iterNr=100;
  for(i=0; i<parNr; i++) {
    par[i]=2.0; delta[i]=0.1;
  }
  ret=powell(par, delta, parNr, 0.002, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  for(i=0; i<parNr; i++) {
    if(fabs(par[i]-1.0)>1.0E-06) {
      if(VERBOSE) {printf("   FAILED: powell() did not reach required minimum\n");}
      return (1);
    }
  }
  
  printf("test_powell(): Rosenbrock's valley with N=6, starting from global min\n");
  parNr=6; iterNr=100;
  for(i=0; i<parNr; i++) {
    par[i]=1.0; delta[i]=0.1;
  }
  ret=powell(par, delta, parNr, 0.002, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  for(i=0; i<parNr; i++) {
    if(fabs(par[i]-1.0)>1.0E-06) {
      if(VERBOSE) {printf("   FAILED: powell() did not reach required minimum\n");}
      return (1);
    }
  }

  
  printf("test_powell(): Rosenbrock's valley with N=6, one parameters fixed\n");
  parNr=6; iterNr=100;
  for(i=0; i<parNr; i++) {
    par[i]=2.0; delta[i]=0.1;
  }
  par[3]=1.0; delta[3]=0.0;
  ret=powell(par, delta, parNr, 0.002, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  for(i=0; i<parNr; i++) {
    if(fabs(par[i]-1.0)>1.0E-06) {
      if(VERBOSE) {printf("   FAILED: powell() did not reach required minimum\n");}
      return (1);
    }
  }

  
  printf("test_powell(): ");
  printf("Rosenbrock's valley with N=6, starting close to local min\n");
  parNr=6; iterNr=100;
  for(i=0; i<parNr; i++) {
    if(i&1) par[i]=0.9; else par[i]=1.1; delta[i]=0.02;
  }
  par[0]=-0.95; delta[0]=0.03;
  if(VERBOSE) {
    i=0; printf("initial parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
  }
  ret=powell(par, delta, parNr, 0.00000001, &iterNr, &f, optfunc_dejong2, NULL, 0);
  if(ret>1) {
    if(VERBOSE) {printf("   FAILED: powell() returned error %d\n", ret);}
    return (1);
  }
  if(ret==1) {
    if(VERBOSE) {printf("   FAILED: powell() did not reach required tolerance\n");}
    return (1);
  }
  if(VERBOSE) {
    printf("powell() iterations: %d\n", iterNr);
    i=0; printf("estimated parameter values: %g", par[i]);
    for(i=1; i<parNr; i++) printf(", %g", par[i]); printf("\n");
    printf("estimated function minimum: %g\n", f);
  }
  if(fabs(f-3.97394)>0.001) {
    if(VERBOSE) {printf("   FAILED: powell() did not find the local minimum\n");}
    return (1);
  }
#endif
/*****************************************************************************/

/*****************************************************************************/
