/** @file tpctacmod.h
 *  @brief Header file for libtpctacmod.
 *  @details libtpctacmod contains functions needed in reading and processing 
 *  PET TAC data for modelling and fitting.
 *  @author Vesa Oikonen
 */
#ifndef _TPCTACMOD_H_
#define _TPCTACMOD_H_
/*****************************************************************************/

/*****************************************************************************/
#include "tpcclibConfig.h"
/*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
/*****************************************************************************/
#include "tpcextensions.h"
#include "tpcli.h"
#include "tpctac.h"
#include "tpcmodels.h"
#include "tpcpar.h"
#include "tpcfunc.h"
/*****************************************************************************/

/*****************************************************************************/
/* litac */
int tacInterpolate(
  TAC *inp, TAC *xinp, TAC *tac, TAC *itac, TAC *iitac, TPCSTATUS *status
);
int tacInterpolateInto(
  TAC *inp, TAC *tac, TAC *itac, TAC *iitac, TPCSTATUS *status
);
int tacInterpolateToEqualLengthFrames(
  TAC *inp, double minfdur, double maxfdur, TAC *tac, TPCSTATUS *status
);
int tacFramesToSteps(TAC *inp, TAC *out, TPCSTATUS *status);
/*****************************************************************************/

/*****************************************************************************/
#if(0)
/* litacrange */
int tacInterpolateCheckStart(
  TAC *inp, TAC *tac, int forgiving, TPCSTATUS *status
);
#endif
/*****************************************************************************/

/*****************************************************************************/
/* tacmodelinput */
int tacFittime(
  TAC *d, double *startTime, double *endTime, int *first, int *last,
  TPCSTATUS *status
);
int tacReadModelingData(
  const char *tissuefile, const char *inputfile1,
  const char *inputfile2, const char *inputfile3,
  double *fitdur, int cutInput, int *fitSampleNr, TAC *tis, TAC *inp,
  TPCSTATUS *status
);
int tacReadReference(
  TAC *tis, const char *reference, TAC *ref, int *refIndex, TPCSTATUS *status
);
int tacReadModelingInput(
  const char *inputfile1, const char *inputfile2, const char *inputfile3,
  TAC *inp, TPCSTATUS *status
);
/*****************************************************************************/

/*****************************************************************************/
/* tacfitplot */
int tacPlotFitSVG(
  TAC *tac1, TAC *tac2, const char *main_title, 
  const double x1, const double x2, const double y1, const double y2, 
  const char *fname, TPCSTATUS *status
);
int tacPlotLineSVG(
  TAC *tac, const char *main_title,
  const double x1, const double x2, const double y1, const double y2,
  const char *fname, TPCSTATUS *status
);
/*****************************************************************************/

/*****************************************************************************/
/* partac */
int tacAllocateWithPAR(TAC *tac, PAR *par, int sampleNr, TPCSTATUS *status);
int parAllocateWithTAC(PAR *par, TAC *tac, int parNr, TPCSTATUS *status);
int tacToPAR(TAC *tac, PAR *par, TPCSTATUS *status);
/*****************************************************************************/

/*****************************************************************************/
#endif /* _TPCTACMOD_H_ */
